import * as fs from "fs";
import * as path from "path";
import atomicWrite from "write-file-atomic";
import has from "lodash.has";
import get from "lodash.get";
import set from "lodash.set";
import unset from "lodash.unset";
class TeenyConf {
    _configPath;
    _defaultConfig;
    _conf;
    constructor(configPath, defaultConfig = {}) {
        if (!configPath)
            throw new TypeError("teenyconf needs a valid configPath");
        this._configPath = path.resolve(configPath);
        this._defaultConfig = defaultConfig;
        // Check if directory exists, creates it if needed
        if (!fs.existsSync(path.dirname(this._configPath))) {
            fs.mkdirSync(path.dirname(this._configPath));
        }
        try {
            // Load the file
            this._conf = JSON.parse(fs.readFileSync(this._configPath).toString());
        }
        catch (err) {
            // console.info(`An error occured when parsing ${_configPath}, fallback on default config`);
            this._conf = this._defaultConfig;
            this.save();
        }
    }
    /**
     * Reload the configuration from file
     */
    reload() {
        this._conf = JSON.parse(fs.readFileSync(this._configPath).toString());
    }
    /**
     * Save current config to its associated file
     * @param  {Boolean} [minify=false] minify the content of the file
     */
    save(minify = false) {
        const output = minify
            ? JSON.stringify(this._conf)
            : JSON.stringify(this._conf, null, " ");
        atomicWrite.sync(this._configPath, output);
    }
    get(key, def) {
        if (key) {
            if (has(this._conf, key))
                return get(this._conf, key);
            return def;
        }
        // Else return everything
        return this._conf;
    }
    /**
     * Set/add key/value pair
     * @param  {String} key key to be updated
     * @param  {any} value new key value
     */
    set(key, value) {
        set(this._conf, key, value);
    }
    /**
     * Delete a key from the configuration
     * @param  {String} key key to be deleted
     */
    delete(key) {
        unset(this._conf, key);
    }
    /**
     * Delete a key from the configuration
     */
    clear() {
        this._conf = {};
    }
    /**
     * Check if a key exists
     * @param  {String} key
     */
    has(key) {
        return has(this._conf, key);
    }
}
export default TeenyConf;
//# sourceMappingURL=teeny-conf.js.map